import React, { useState, useEffect } from "react";
import { Spin } from 'antd';
import "@wangeditor/editor/dist/css/style.css";
import { Editor, Toolbar } from "@wangeditor/editor-for-react";
import { DomEditor, Boot } from "@wangeditor/editor";
import { customUploadImage, customBrowseAndUploadVideo } from "./util";
import videoToHtmlConf from "./plugins/video/videoToHtmlConf";
import imageToHtmlConf from "./plugins/image/imageToHtmlConf";

Boot.registerElemToHtml(videoToHtmlConf);
Boot.registerElemToHtml(imageToHtmlConf);

// 工具栏配置参考
// https://www.cnblogs.com/-roc/p/16400965.html

const defaultStyle = {
  border: "1px solid #ccc",
  zIndex: 100,
  marginTop: "15px",
};

function MyEditor (props) {
  const style = React.useMemo(
    () => ({ ...defaultStyle, ...(props.style || {}) }),
    [props.style]
  );

  const {
    value = "",
    onChange = (e) => {
      setHtml(e);
    },
    toolbarKeys,
    toolbarConfig = {},
    editorConfig = {
      placeholder: "请输入内容...",
    },
    readonly = false,
    mode = "default",
    height = 500,
  } = props;
  const [editor, setEditor] = useState(null); // 存储 editor 实例
  const [html, setHtml] = useState("");
  const [loading, setLoading] = useState(false);

  const toolbarConf = React.useMemo(() => {
    return toolbarKeys
      ? {
        toolbarKeys,
        ...toolbarConfig,
      }
      : toolbarConfig;
  }, [toolbarConfig, toolbarKeys]);

  const editorConf = React.useMemo(() => {
    const menuConf = editorConfig.MENU_CONF;

    return {
      ...editorConfig,
      MENU_CONF: {
        uploadImage: { customUpload: customUploadImage },
        uploadVideo: { customBrowseAndUpload: customBrowseAndUploadVideo(setLoading) },
        ...menuConf,
        fontSize: {
          fontSizeList: [
            "10px",
            "11px",
            "12px",
            "13px",
            "14px",
            "15px",
            "16px",
            "18px",
            "20px",
            "24px",
            "28px",
            "40px",
            "64px",
            "72px",
          ]
        },
        lineHeight: {
          // 从1 到 2.5
          lineHeightList: (new Array(16)).fill().map((_, inx) => `${1 + inx / 10}`),
        }
      },
    };
  }, [editorConfig]);

  // 模拟 ajax 请求，异步设置 html
  useEffect(() => {
    setHtml(value || "");
  }, [value]);

  // 及时销毁 editor
  useEffect(() => {
    if (editor) {
      const toolbar = DomEditor.getToolbar(editor);
      const curToolbarConfig = toolbar.getConfig();
      // console.log(curToolbarConfig.toolbarKeys); // 当前菜单排序和分组
      // console.log(editor?.getAllMenuKeys());
    }

    return () => {
      if (editor == null) return;
      editor.destroy();
      setEditor(null);
    };
  }, [editor]);

  function insertText () {
    if (editor == null) return;
    editor.insertText(" hello ");
  }

  function printHtml () {
    if (editor == null) return;
    console.log(editor.getHtml());
  }

  const handleChange = (editor) => {
    let raw = editor.getHtml();

    if (raw) {
      raw = raw.replaceAll('<p>', '<p class="rich-paragraph">');
    }

    if (raw !== "<p><br></p>") {
      onChange(raw);
    } else {
      onChange("");
    }
  };

  return !readonly ? (
    <div className={props.className} style={style}>
      <Toolbar
        editor={editor}
        defaultConfig={toolbarConf}
        mode={mode}
        style={{ borderBottom: "1px solid #ccc" }}
      />
      <Spin spinning={loading}>
        <Editor
          defaultConfig={editorConf}
          value={html}
          onCreated={setEditor}
          onChange={handleChange}
          mode={mode}
          style={{ height: `${height}px` }}
        />
      </Spin>
    </div>
  ) : (
    <div dangerouslySetInnerHTML={{ __html: value }}></div>
  );
}

export default MyEditor;
